/*! \file RFM12_STXETX.c \brief RF communication with STX/ETX protocol. */
//*****************************************************************************
//
// File Name	: 'RFM12_STXETX.c'
// Title		: RFM12 RF communication with STX/ETX protocol
// Author		: Claude Schwarz
// Created		: 5/20/2007
// Revised		: 5/20/2007
// Version		: 0.1
// Target MCU	: atmega8
//
// Includes parts from Benedikt K.s RFM12 example
// http://www.mikrocontroller.net/topic/67273#new
//
// Includes parts from Pascal Stangs Procyon AVRlib
// http://hubbard.engr.scu.edu/avr/avrlib/docs/html/group__stxetx.html
//
// Includes parts from Peter Fleurys uart lib
// http://jump.to/fleury
//
// Includes parts from JBeckers RFM12 example
// http://www.mikrocontroller.net/topic/67273#new
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include "global.h"
#include "uart.h"
#include "rf12.h" 
#include "stxetx.h"
#include "buffer.h"
#include <util/delay.h>

#define 	UART_RX_BUFFER_SIZE   32		
#define 	UART_TX_BUFFER_SIZE   32   
#define F_CPU 14745600UL

//Functions
void send(void);
void receive(void);
cBuffer* rfGetRxBuffer(void); //rfGetRxBuffer function
void sendrf(unsigned char rfdata);

//Variables
unsigned char receiverf(void);
unsigned char TxPacket[10];
unsigned int packet = 0;
unsigned char rfdata_array[10];
static char rfRxData[10]; 						// allocate buffer in ram
cBuffer rfRxBuffer;		  						// assign cBuffer to rfRXBuffer
unsigned char* dataPtr;


int main(void)
{
	uart_init(UART_BAUD_SELECT(1200, F_CPU));	// Init Uart 1200bps
	rf12_init();								// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf12_setfreq(RF12FREQ(433.92));				// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf12_setbandwidth(4, 1, 4);					// 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
	rf12_setbaud(14400);						// 19200 baud
	rf12_setpower(0, 6);						// 1mW Ausgangangsleistung, 120kHz Frequenzshift
	bufferInit(&rfRxBuffer, rfRxData, 7);		// Init ringbuffer 
	bufferFlush (&rfRxBuffer);					// flush ringbuffer
	stxetxInit(sendrf);							// Init stxetx lib 
	
	sei();										// Enable Interrupts
	uart_puts_P("Empfaenger laeuft !\n");


	for (;;)
	{
		receive();
		send();
	}
}


void receive(void)
{	if(stxetxProcess(rfGetRxBuffer()))
		{
		dataPtr = stxetxGetRxPacketData();
		uart_putc( dataPtr[0] );
		bufferFlush (&rfRxBuffer);
		}
}

void send(void)
{	//unsigned char test[]="Dies ist ein 433MHz Test !!!!!!\n";	
	//rf12_txdata(test,32);
	unsigned int c;
	c = uart_getc();
	
        if ( c & UART_NO_DATA )				//Check if data to send is in the ringbuffer
        {
		}
	
	else
        {
		TxPacket[0] = c;
		stxetxSend(0, 0x55, 1, TxPacket);  //Format:[STX][status][type][length][user data...][checksum][ETX]
		} 									//		         0	   0x55	    1	 TxPacket
											//        FIX  USER	   USER   USER	  USER			FIX      FIX
}

void sendrf(unsigned char rfdata)
{		
	rfdata_array[packet]=rfdata;
	packet++;
	if (packet==7)							// ToDo: Fixed Packet length
		{
		rf12_txdata(rfdata_array,9);
		//for (unsigned char i=0; i<7; i++)//Debug purpose
		//	uart_putc(rfdata_array[i]);	
		packet=0;
		}
	else
		{}

}

cBuffer* rfGetRxBuffer(void)
{
	bufferFlush (&rfRxBuffer);
	unsigned char Buffer[10];
	unsigned int chars = rf12_rxdata(Buffer,10);
	if(chars > 0){
		for (unsigned char i=0; i<10; i++){
		bufferAddToEnd(&rfRxBuffer, Buffer[i]);}
	return &rfRxBuffer;}
	else{return 0;}
}
